/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/05/29 11:48:01
 */

import { nextTick, css, html, Component, bind } from 'wkit'
import '../icon/index.js'
import '../option/index.js'

class Dropdown extends Component {
  static props = {}
  static styles = [
    css`
      :host {
        display: inline-flex;
        min-width: 108px;
        height: 32px;
        font-size: 14px;
        user-select: none;
        -moz-user-select: none;
        color: var(--color-dark-1);
      }

      .dropdown {
        position: relative;
        display: flex;
        width: 100%;
        height: 100%;
      }

      .default {
        display: flex;
        align-items: center;
        justify-content: center;
        width: 100%;
        height: 100%;
        padding: 0 8px;

        .arrow {
          --wc-icon-size: 12px;
          margin-left: 6px;
          transform: rotate(-90deg);
        }
      }
    `,
    // 尺寸
    css`
      @use 'sass:map';
      $sizes: (
        s: (
          w: 52px,
          h: 20px,
          f: 12px
        ),
        m: (
          w: 72px,
          h: 24px,
          f: 12px
        ),
        l: (
          w: 108px,
          h: 32px,
          f: 14px
        ),
        xl: (
          w: 132px,
          h: 36px,
          f: 14px
        ),
        xxl: (
          w: 160px,
          h: 44px,
          f: 14px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          min-width: map.get($v, 'w');
          height: map.get($v, 'h');
          font-size: map.get($v, 'f');
        }
      }
    `,

    css`
      .dropdown-list {
        overflow: hidden;
        position: absolute;
        top: 32px;
        padding: 8px 0;
        border-radius: 3px;
        background: #fff;
        box-shadow: 0 0 12px rgba(0, 0, 0, 0.1);
      }
    `
  ]

  mounted() {}

  render() {
    return html`
      <div class="dropdown">
        <div class="default">
          <slot></slot>
          <wc-icon class="arrow" name="left"></wc-icon>
        </div>

        <div class="dropdown-list">
          <slot name="option"></slot>
        </div>
      </div>
    `
  }
}

Dropdown.reg('dropdown')
