/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/16 18:05:43
 */
import { css, html, Component, bind, unbind, nextTick } from 'wkit'
import '../icon/index.js'

class Passwd extends Component {
  static props = {
    value: 'str!',
    autofocus: false,
    readonly: false,
    disabled: false,
    placeholder: 'str!',
    lazy: 'num!0' // 并发拦截时间, 单位毫秒
  }

  static styles = [
    css`
      :host {
        overflow: hidden;
        display: inline-flex;
        min-width: 188px;
        user-select: none;
        -moz-user-select: none;
        color: var(--color-dark-1);
        border-radius: 3px;
        cursor: text;
        transition: box-shadow 0.15s linear;
      }

      .label {
        flex: 1;
        display: flex;
        justify-content: center;
        align-items: center;
        height: 32px;
        font-size: 14px;
        border: 1px solid var(--wc-passwd-border-color, var(--color-grey-2));
        border-radius: inherit;
        background: var(--wc-passwd-background, #fff);
        color: inherit;
        cursor: inherit;

        input {
          flex: 1;
          min-width: 36px;
          width: 0;
          height: 100%;
          padding: 0 4px 0 8px;
          border: 0;
          border-radius: inherit;
          color: inherit;
          font: inherit;
          background: none;
          outline: none;
          box-shadow: none;
          cursor: inherit;

          &::placeholder {
            color: var(--color-grey-2);
          }
        }
      }
      :host([round]) .label input {
        padding: 0 6px 0 20px;
      }
      :host(:focus-within) {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
    `,
    css`
      .label {
        .icon {
          --wc-icon-size: 16px;
          margin: 0 8px 0 4px;
          color: var(--color-grey-3);
          cursor: pointer;
        }
      }
    `,
    // 尺寸
    css`
      @use 'sass:map';
      $sizes: (
        m: (
          w: 128px,
          h: 24px,
          f: 12px
        ),
        // l default
        xl:
          (
            w: 224px,
            h: 36px,
            f: 14px
          ),
        xxl: (
          w: 288px,
          h: 44px,
          f: 14px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          min-width: map.get($v, 'w');
          .label {
            height: map.get($v, 'h');
          }

          @if $s == 'xxl' {
            .icon {
              --wc-icon-size: 18px;
            }
          }
        }
      }
    `,
    css`
      :host([disabled]),
      :host([readonly]) {
        cursor: default;
        .icon {
          cursor: default;
        }
      }
      :host([disabled]) {
        cursor: not-allowed;

        .label {
          border-color: var(--color-grey-1);
          background: var(--color-plain-1);
          opacity: 0.6;
        }
      }
    `
  ]

  #type = 'password'

  mounted() {
    if (this.autofocus) {
      nextTick(_ => this.$refs.input.focus())
    }
  }

  iconClick(ev) {
    if (this.readOnly || this.disabled) {
      return
    }
    this.#type = this.#type === 'password' ? '' : 'password'
    this.$requestUpdate()
  }

  handleInput(ev) {
    this.value = ev.target.value
  }

  handleChange() {
    this.$emit('change')
  }

  render() {
    return html`
      <div class="label">
        <slot class="prepend" name="prepend"></slot>
        <input
          spellcheck="false"
          ref="input"
          placeholder=${this.placeholder}
          @input=${this.handleInput}
          @change=${this.handleChange}
          :readOnly=${this.readOnly}
          :disabled=${this.disabled}
          :type=${this.#type}
          :value=${this.value}
        />
        <wc-icon
          class="icon"
          @click=${this.iconClick}
          :name=${this.#type === 'password' ? 'eye' : 'eye-off'}
        ></wc-icon>
      </div>
    `
  }
}

Passwd.reg('passwd')
