/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/21 16:14:10
 */

import { nextTick, css, html, Component } from 'wkit'

class Radio extends Component {
  static props = {
    value: {
      type: String,
      default: '',
      attribute: false,
      observer() {
        this.#updateChildrenStat()
      }
    },
    disabled: false,
    readonly: false
  }

  static styles = css`
    :host {
      display: inline-flex;
      flex-wrap: wrap;
      align-items: center;
    }
  `

  mounted() {
    this.$on('child-change', ev => {
      ev.stopPropagation()
      this.value = ev.value
      this.$emit('input', { data: ev.value })
      this.$emit('change', { data: ev.value })
    })
    this.#updateChildrenStat(true)
  }

  #updateChildrenStat(checkAll) {
    Array.from(this.children).forEach(it => {
      if (it.tagName === 'WC-RADIO') {
        if (it.root) {
          if (checkAll) {
            it.disabled = this.disabled
            it.readOnly = this.readOnly
          }

          if (it.value === this.value) {
            it.checked = true
          } else {
            it.checked = false
          }
        }
      } else {
        it.remove()
      }
    })
  }
}

class RadioItem extends Component {
  static props = {
    value: 'str!',
    checked: false,
    disabled: false,
    readOnly: false
  }

  static styles = [
    css`
      :host {
        display: inline-flex;
        cursor: pointer;

        label {
          display: flex;
          align-items: center;
          padding-right: 16px;
          line-height: 1;
          white-space: nowrap;
          align-items: center;
          font-size: 14px;
          cursor: inherit;
          outline: none;
          color: var(--color-dark-1);
          -webkit-user-select: none;
          user-select: none;
        }

        .dot {
          display: flex;
          justify-content: center;
          align-items: center;
          width: 14px;
          height: 14px;
          margin-right: 4px;
          border: 1px solid var(--color-grey-2);
          border-radius: 50%;
          background: #fff;
          transition: box-shadow 0.15s linear;

          &::after {
            display: block;
            visibility: hidden;
            width: 6px;
            height: 6px;
            border-radius: 50%;
            background: var(--color-dark-1);
            content: '';
            transform: scale(0);
            transition: transform 0.15s linear;
          }
        }
        &:host([checked]) .dot::after {
          visibility: visible;
          transform: scale(1);
        }
      }
    `,

    css`
      :host(:focus-within) .dot {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
    `,
    // 尺寸
    css`
      @use 'sass:math';
      @use 'sass:map';
      $sizes: (
        s: (
          h: 20px,
          f: 12px
        ),
        m: (
          h: 24px,
          f: 12px
        ),
        xl: (
          h: 36px,
          f: 16px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          height: map.get($v, 'h');

          .dot {
            width: map.get($v, 'f');
            height: map.get($v, 'f');

            &::after {
              width: math.div(map.get($v, 'f'), 2);
              height: math.div(map.get($v, 'f'), 2);
            }
          }
        }
      }
    `,
    // 配色
    css`
      $colors: (
        primary: 'teal',
        info: 'blue',
        success: 'green',
        warning: 'orange',
        danger: 'red'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']) {
          label {
            color: var(--color-#{$c}-2);
          }

          .dot {
            border-color: var(--color-#{$c}-2);

            &::after {
              background: var(--color-#{$c}-2);
            }
          }

          &:host(:focus-within) .dot {
            box-shadow: 0 0 0 2px var(--color-#{$c}-a);
          }
        }
      }
    `,
    // 状态
    css`
      :host([readonly]),
      :host([disabled]) {
        opacity: 0.6;
      }
      :host([disabled]) {
        cursor: not-allowed;
      }
    `
  ]

  #toggleCheck(ev) {
    if (this.disabled || this.readOnly || this.checked) {
      return
    }

    ev.stopPropagation()

    this.checked = true

    if (this.inGroup) {
      this.parentNode.$emit('child-change', { value: this.value })
    } else {
      this.$emit('input')
      this.$emit('change')
    }
  }

  #click(ev) {
    if (ev.type === 'click' || ev.keyCode === 32) {
      this.#toggleCheck(ev)
    }
  }

  mounted() {
    if (this.parentNode?.tagName === 'WC-RADIO-GROUP') {
      this.inGroup = true
    }
  }

  render() {
    return html` <label
      tabindex=${this.disabled || this.readOnly ? 'none' : 0}
      @click=${this.#click}
      @keydown=${this.#click}
    >
      <span class="dot"></span>
      <slot></slot>
    </label>`
  }
}

Radio.reg('radio-group')
RadioItem.reg('radio')
