/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/21 16:14:10
 */

import { nextTick, css, html, Component, classMap } from 'wkit'

class Switch extends Component {
  static props = {
    value: false,
    inactiveText: '',
    activeText: '',
    inlineText: false,
    disabled: false,
    readOnly: false
  }

  static styles = [
    css`
      :host {
        display: inline-flex;
        cursor: pointer;

        label {
          display: flex;
          align-items: center;
          height: 32px;
          padding-right: 8px;
          line-height: 1;
          font-size: 14px;
          white-space: nowrap;
          color: var(--color-dark-1);
          cursor: inherit;
          outline: none;
          -webkit-user-select: none;
          user-select: none;

          &.open .switch {
            flex-direction: row-reverse;
            background: var(--color-grey-3);
          }
        }

        .switch {
          display: flex;
          align-items: center;
          min-width: 32px;
          height: 18px;
          padding: 0 2px;
          margin-right: 5px;
          border-radius: 16px;
          font-family: Tahoma, Verdana, Helvetica, Arial, sans-serif;
          font-size: 12px;
          background: var(--color-plain-3);
          transition: box-shadow 0.2s ease, background 0.2s ease;
        }
        .dot {
          display: block;
          width: 14px;
          height: 14px;
          border-radius: 50%;
          background: #fff;
          content: '';
        }
        .text {
          padding: 0 3px;
          background: none;
          color: #fff;
        }
      }

      :host(:focus-within) .switch {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
    `,

    // 尺寸
    css`
      @use 'sass:map';
      @use 'sass:math';
      $sizes: (
        s: (
          hd: 14px,
          h: 20px,
          f: 10px
        ),
        m: (
          hd: 16px,
          h: 24px,
          f: 12px
        ),
        xl: (
          hd: 20px,
          h: 36px,
          f: 14px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          label {
            height: map.get($v, 'h');
            font-size: math.max(map.get($v, 'f'), 12px);
          }

          .switch {
            min-width: map.get($v, 'hd') * 2 - 4;
            height: map.get($v, 'hd');
            padding: 0 #{math.div((map.get($v, 'hd') - map.get($v, 'f')), 2)};
          }
          .dot {
            width: map.get($v, 'f');
            height: map.get($v, 'f');
          }
        }
      }
    `,
    // 配色
    css`
      $colors: (
        primay: 'teal',
        info: 'blue',
        success: 'green',
        warning: 'orange',
        danger: 'red'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']) {
          .open .switch {
            background: var(--color-#{$c}-1);
          }

          &:host(:focus-within) .switch {
            box-shadow: 0 0 0 2px var(--color-#{$c}-a);
          }
        }
      }
    `,
    // 状态
    css`
      :host([readonly]),
      :host([disabled]) {
        opacity: 0.6;
      }
      :host([disabled]) {
        cursor: not-allowed;
      }
    `
  ]

  get #text() {
    return this.value ? this.activeText : this.inactiveText
  }

  #toggleCheck(ev) {
    if (this.disabled || this.readOnly) {
      return
    }

    ev.stopPropagation()

    this.value = !this.value
    let data = { value: this.value }

    this.$emit('input', data)
    this.$emit('change', data)
  }

  #click(ev) {
    if (ev.type === 'click' || ev.keyCode === 32) {
      ev.preventDefault()
      this.#toggleCheck(ev)
    }
  }

  render() {
    return html` <label
      class=${classMap({ open: this.value })}
      tabindex=${this.disabled || this.readOnly ? 'none' : 0}
      @click=${this.#click}
      @keydown=${this.#click}
    >
      <span class="switch">
        <i class="dot"></i>
        <mark class="text">${this.inlineText ? this.#text : ''}</mark>
      </span>
      <slot>${this.inlineText ? '' : this.#text}</slot>
    </label>`
  }
}

Switch.reg('switch')
