/**
 * {markd组件}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/06 15:17:25
 */

import { css, html, raw, Component, bind, styleMap, classMap } from 'wkit'

import md2html from './core.js'
import '../code/index.js'
import '../form/checkbox.js'

export default md2html

class Markd extends Component {
  static props = {
    code: { type: String, default: '', attribute: false }
  }

  static styles = css`
    :host {
      display: block;
      line-height: 1.5;
      color: var(--color-dark-1);
      font-size: 14px;
    }
    a {
      text-decoration: underline;
      color: var(--color-teal-2);
    }
    a:hover {
      color: var(--color-teal-1);
      text-decoration: none;
    }

    em,
    del {
      color: var(--color-grey-2);
    }
    strong,
    strong em,
    strong {
      color: var(--color-dark-3);
    }
    a {
      strong,
      em {
        color: inherit;
      }
    }
    em,
    strong,
    del {
      padding: 0 2px;
    }
    p {
      margin: 12px 0;
    }
    img {
      max-width: 100%;
    }

    blockquote.md-quote {
      margin: 10px 0;
      padding: 5px 10px;
      line-height: 1.5;
      border-left: 5px solid var(--color-teal-1);
      background: #f2faf7;
      color: var(--color-grey-2);
      p {
        margin: 0;
      }
      > blockquote {
        margin: 5px 0;
        padding: 0 10px;
      }
    }

    hr {
      height: 1px;
      margin: 30px 0;
      border: 0;
      border-top: 1px dashed var(--color-plain-3);
    }
    ol {
      margin-left: 1em;
      list-style: decimal outside none;
    }
    ul {
      margin-left: 1em;
      list-style: disc outside none;
    }
    li {
      margin: 0.5em 0;
    }
    li ol {
      margin-left: 1em;
    }
    li ul {
      margin-left: 1em;
      list-style-type: circle;
    }
    li ol ul,
    li ul ul {
      list-style-type: square;
    }

    h1,
    h2,
    h3,
    h4,
    h5,
    h6 {
      margin: 15px 0;
      line-height: 2;
      font-weight: bold;
      font-size: 16px;

      code.inline {
        background: none;
      }
      a {
        text-decoration: none;
        color: #333;
      }
    }
    h3,
    h4,
    h5,
    h6 {
      a {
        &::before {
          content: '∮ ';
          color: var(--color-teal-1);
          font-weight: normal;
        }
      }
    }
    h1 {
      margin: 0 0 30px;
      font-size: 24px;
      text-align: center;
    }
    h2 {
      margin: 20px 0;
      font-size: 22px;
      border-bottom: 1px solid var(--color-plain-2);
    }
    h3 {
      margin: 20px 0 15px;
      font-size: 20px;
    }
    h4 {
      font-size: 18px;
    }

    table {
      width: 100%;
      border-spacing: 0;
      border-collapse: collapse;

      tr {
        background: #fff;
      }
      thead tr {
        background: var(--color-plain-1);
      }
      th,
      td {
        padding: 6px 13px;
        border: 1px solid var(--color-plain-2);
        vertical-align: middle;
      }
      th {
        font-weight: bold;
      }
      tr:nth-child(2n) {
        background: #fcfdff;
      }
    }

    code.inline {
      display: inline;
      margin: 0 2px;
      padding: 0 2px;
      color: var(--color-red-1);
      background: var(--color-plain-1);
      border-radius: 2px;
      font-family: Menlo, Monaco, Consolas, 'Courier New', monospace;
    }
  `

  render() {
    return html`<div class="container">${raw(md2html(this.code))}</div>`
  }
}

Markd.reg('markd')
