/**
 * 一些公共的东西
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/10/12 18:23:23
 */

const PLACEHOLDER = '在此输入文本'

const ELEMS = {
  a: function (str, attr, inner) {
    let href = attr.match(attrExp('href'))
    let title = attr.match(attrExp('title'))
    let tar = attr.match(attrExp('target'))
    let attrs = ''

    href = (href && href[1]) || null
    title = (title && title[1]) || null
    tar = tar && tar[1]

    if (!href) {
      return inner || href
    }

    href = href.replace('viod(0)', '').replaceAll('&amp;', '&')
    attrs = tar ? `target=${tar}` : ''
    attrs += title ? `;title=${title}` : ''

    return `[${inner || href}](${href}${attrs ? ` "${attrs}"` : ''})`
  },
  em: function (str, attr, inner) {
    return (inner && '_' + inner + '_') || ''
  },
  strong: function (str, attr, inner) {
    return (inner && '**' + inner + '**') || ''
  },
  pre: function (str, attr, inner) {
    inner = inner.replace(/<[/]?code>/g, '')
    return '\n\n```\n' + inner + '\n```\n'
  },
  code: function (str, attr, inner) {
    return (inner && '`' + inner + '`') || ''
  },
  blockquote: function (str, attr, inner) {
    return '> ' + inner.trim()
  },
  img: function (str, attr, inner) {
    var src = attr.match(attrExp('src')),
      alt = attr.match(attrExp('alt'))

    src = (src && src[1]) || ''
    alt = (alt && alt[1]) || ''

    return '![' + alt + '](' + src + ')'
  },
  p: function (str, attr, inner) {
    return inner ? '\n' + inner : ''
  },
  br: '\n',
  'h([1-6])': function (str, level, attr, inner) {
    let h = '#'.repeat(level)
    return '\n' + h + ' ' + inner + '\n'
  },
  hr: '\n\n---\n\n'
}

export const DEFAULT_TOOLS = [
  'header',
  'quote',
  'bold',
  'italic',
  'through',
  'list',
  'order',
  'line',
  'code',
  'codeblock',
  'table',
  'link',
  'image',
  'attach',
  'fullscreen',
  'preview'
]

const LI_EXP = /<(ul|ol)>(?:(?!<ul|<ol)[\s\S])*?<\/\1>/gi

function trim(str, sign) {
  return str.replace(new RegExp('^' + sign + '|' + sign + '$', 'g'), '')
}

// html标签的属性正则
function attrExp(field, flag = 'i') {
  return new RegExp(field + '\\s?=\\s?["\']?([^"\']*)["\']?', flag)
}

// 生成html标签的正则
function tagExp(tag, open) {
  var exp = ''
  if (['br', 'hr', 'img'].indexOf(tag) > -1) {
    exp = '<' + tag + '([^>]*?)\\/?>'
  } else {
    exp = '<' + tag + '([^>]*?)>([\\s\\S]*?)<\\/' + tag + '>'
  }
  return new RegExp(exp, 'gi')
}

/**
 * html转成md
 */
export function html2md(str) {
  try {
    str = decodeURIComponent(str)
  } catch (err) {}

  str = str
    .replace(/\t/g, '  ')
    .replace(/<\/?(meta|link|script)[^>]*?>/g, '')
    .replace(/<!--[\w\W]*?-->/g, '')
    .replace(/<xml[^>]*?>[\w\W]*?<\/xml>/g, '')
    .replace(/<style>[\w\W]*?<\/style>/g, '')
    .replace(attrExp('class', 'g'), '')
    .replace(attrExp('style', 'g'), '')
    .replace(/<a[^>]*? href\s?=\s?["']?([^"']*)["']?[^>]*?>/g, '<a href="$1">')
    .replace(
      /<img[^>]*? src\s?=\s?["']?([^"']*)["']?[^>]*?>/g,
      '<img src="$1">'
    )
    .replace(/<(?!a |img )(\w+) [^>]*>/g, '<$1>')
    .replace(/<svg[^>]*>.*?<\/svg>/g, '{svg not support}')

  // log(str)
  for (let i in ELEMS) {
    let cb = ELEMS[i]
    let exp = tagExp(i)

    if (i === 'blockquote') {
      while (str.match(exp)) {
        str = str.replace(exp, cb)
      }
    } else {
      str = str.replace(exp, cb)
    }

    // 对另外3种同类标签做一次处理
    if (i === 'p') {
      exp = tagExp('div')
      str = str.replace(exp, cb)
    }
    if (i === 'em') {
      exp = tagExp('i')
      str = str.replace(exp, cb)
    }
    if (i === 'strong') {
      exp = tagExp('b')
      str = str.replace(exp, cb)
    }
  }

  while (str.match(LI_EXP)) {
    str = str.replace(LI_EXP, function (match) {
      match = match.replace(
        /<(ul|ol)>([\s\S]*?)<\/\1>/gi,
        function (m, t, inner) {
          let li = inner.split('</li>')
          li.pop()

          for (let i = 0, len = li.length; i < len; i++) {
            let pre = t === 'ol' ? i + 1 + '. ' : '* '
            li[i] =
              pre +
              li[i]
                .replace(/\s*<li>([\s\S]*)/i, function (m, n) {
                  n = n.trim().replace(/\n/g, '\n  ')
                  return n
                })
                .replace(/<[\/]?[\w]*[^>]*>/g, '')
          }
          return li.join('\n')
        }
      )
      return '\n' + match.trim()
    })
  }
  str = str

    .replace(/<[\/]?[\w]*[^>]*>/g, '')
    .replace(/```([\w\W]*)```/g, function (str, inner) {
      inner = inner
        .replace(/&amp;/g, '&')
        .replace(/&lt;/g, '<')
        .replace(/&gt;/g, '>')
      return '```' + inner + '```'
    })
  return str
}

export const Addon = {
  header() {
    this.$refs.header.classList.add('fadein')
  },

  h(level) {
    var wrap = this.selection(true) || PLACEHOLDER
    wrap = wrap.replace(/^(#+ )?/, '#'.repeat(level) + ' ')
    this.insert(wrap, true)
  },

  quote() {
    var wrap = this.selection(true) || PLACEHOLDER
    wrap = wrap.replace(/^(>+ )?/, '> ')

    this.insert(wrap, true)
  },

  bold() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '\\*\\*')
    wrap = wrap === unwrap ? `**${wrap}**` : unwrap
    this.insert(wrap, true)
  },

  italic() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '_')
    wrap = wrap === unwrap ? `_${wrap}_` : unwrap
    this.insert(wrap, true)
  },

  through() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '~~')
    wrap = wrap === unwrap ? `~~${wrap}~~` : unwrap
    this.insert(wrap, true)
  },

  list() {
    var wrap = this.selection(true) || PLACEHOLDER

    wrap = wrap.replace(/^([+\-*] )?/, '+ ')
    this.insert(wrap, true)
  },

  order() {
    var wrap = this.selection(true) || PLACEHOLDER

    wrap = wrap.replace(/^(\d+\. )?/, '1. ')
    this.insert(wrap, true)
  },

  line() {
    this.insert('\n\n---\n\n', false)
  },

  code() {
    var wrap = this.selection() || PLACEHOLDER
    var unwrap = trim(wrap, '`')
    wrap = wrap === unwrap ? `\`${wrap}\`` : unwrap
    this.insert(wrap, true)
  },

  codeblock() {
    this.insert('\n```language\n\n```\n')
  },

  table() {
    this.$refs.table.classList.add('fadein')
  },

  link() {
    this.$refs.link.classList.add('fadein')
  },

  fullscreen() {
    this.classList.toggle('fullscreen')
  },
  preview() {
    this.previewEnabled = !this.previewEnabled
    this.$refs.view.classList.toggle('active')
    if (this.previewEnabled) {
      this.$refs.view.code = this.value
    }
  }
}
