/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/04/18 09:38:01
 */

import { css, html, Component } from 'wkit'
import '../form/button.js'

const LAYOUT_DICT = {
  home: e =>
    html`<wc-button
      data-act="1"
      disabled=${e}
      class="item"
      icon="dbl-left"
    ></wc-button>`,
  end: e =>
    html`<wc-button
      data-act="end"
      disabled=${e}
      class="item"
      icon="dbl-right"
    ></wc-button>`,
  prev: e =>
    html`<wc-button
      data-act="prev"
      disabled=${e}
      class="item"
      icon="left"
    ></wc-button>`,
  next: e =>
    html`<wc-button
      data-act="next"
      disabled=${e}
      class="item"
      icon="right"
    ></wc-button>`,
  curr: n => html`<span class="curr item">${n}</span>`,
  info: (t, p) => html`<span class="item">共 ${t}条, ${p}页</span>`,
  jump(n) {
    return html`<section class="item jump">
      <span>Go to</span>
      <input ref="input" @keydown=${this.__gotoPage} maxlength="6" value=${n} />
    </section>`
  }
}

// 计算页码
function calculate(curr, total) {
  let arr = []
  let fixed = 0
  let half = curr < 3 ? 6 - curr : 2 // 中间页码

  // 总页码小于2
  if (total < 2) {
    arr.push({ to: curr, txt: curr })
    return arr
  }

  // 当前页面比半数多时, 前面的用省略号代替
  if (curr - half > 1 && total > 5) {
    let to = curr - 2 * half
    to = to < 1 ? 1 : to
    arr.push({ to, txt: '...' })
  }

  if (total - curr < half) {
    fixed = half - total + curr
  }

  // 把页码拆成2部分来
  for (let i = curr - half - fixed; i < curr + half + 1 && i <= total; i++) {
    if (i > 0) {
      arr.push({ to: i, txt: i })
    }
  }

  // 总页码太多时, 以省略号代替
  if (curr + half < total) {
    let to = curr + 2 * half
    to = to > total ? total : to
    arr.push({ to, txt: '...' })
  }

  return arr
}

class Pager extends Component {
  static props = {
    layout: 'str!home, prev, pages, next, end',
    total: {
      type: Number,
      default: 0,
      attribute: false,
      observer(v) {
        this.totalpage = Math.ceil(v / this.pagesize) || 1
      }
    },
    totalpage: {
      type: Number,
      default: 1,
      attribute: false,
      observer(v) {
        if (v < 1) {
          v = 1
        }
        if (this.page > v) {
          this.page = v
        }
      }
    },
    page: {
      type: Number,
      default: 1,
      attribute: false,
      observer(v) {
        if (v > this.totalpage) {
          this.page = this.totalpage
        } else if (v < 1) {
          this.page = 1
        }
      }
    },
    pagesize: {
      type: Number,
      default: 20,
      attribute: false,
      observer(v) {
        if (this.total > 0) {
          this.totalpage = Math.ceil(this.total / v)
        }
      }
    }
  }

  static styles = [
    css`
      :host {
        display: block;
        line-height: 1;
        font-size: 14px;
        user-select: none;
        -moz-user-select: none;
        color: var(--color-dark-1);

        .layout {
          display: flex;
          justify-content: center;
          align-items: center;
          margin: 10px auto;
          --wc-button-icon-size: 10px;
          --wc-button-border-color: none;
          --wc-button-background: var(--color-plain-1);
          --wc-button-color-hover: var(--color-teal-1);
        }

        .item {
          min-width: 32px;
          width: auto;
          margin: 0 5px;
          border-radius: 3px;

          &.curr {
            display: inline-flex;
            align-items: center;
            justify-content: center;
            height: 32px;
            background: var(--color-teal-1);
            color: #fff;
          }

          &.jump {
            display: flex;
            align-items: center;
          }
        }

        input {
          width: 48px;
          height: 24px;
          padding: 0 4px;
          margin-left: 4px;
          border: 1px solid var(--color-grey-2);
          border-radius: 3px;
          text-align: center;
          outline: none;
          transition: box-shadow 0.15s linear;

          &:focus {
            box-shadow: 0 0 0 2px var(--color-plain-a);
          }
        }
      }
    `,

    css``
  ]

  #handleBtnClick(ev) {
    let elem = ev.target
    if (elem.tagName === 'WC-BUTTON') {
      let { page, totalpage } = this
      let num = elem.dataset.act

      if (elem.disabled) {
        return
      }

      switch (num) {
        case 'prev':
          num = page - 1
          break

        case 'next':
          console.log(num, page)
          num = page + 1
          break

        case 'end':
          num = totalpage
          break
      }

      if (+num === page) {
        return
      }

      this.page = num
      this.$refs.input && (this.$refs.input.value = num)
      this.$emit('page-change', { data: num })
    }
  }

  __gotoPage(ev) {
    if (ev.keyCode === 13) {
      let n = +ev.target.value
      if (n === n) {
        this.page = n
        this.$emit('page-change', { data: n })
      }
      ev.target.value = this.page
    }
  }

  render() {
    let layout = this.layout.split(',').map(s => s.trim())
    let { page, total, totalpage } = this

    return html`
      <div class="layout" @click=${this.#handleBtnClick}>
        ${layout.map(n => {
          switch (n) {
            case 'pages':
              return calculate(page, totalpage).map(it =>
                page === it.to
                  ? LAYOUT_DICT.curr(page)
                  : html`<wc-button class="item" data-act="${it.to}"
                      >${it.txt}</wc-button
                    >`
              )

            case 'home':
              return LAYOUT_DICT.home(page === 1)

            case 'prev':
              return LAYOUT_DICT.prev(page === 1)

            case 'next':
              return LAYOUT_DICT.next(page >= totalpage)

            case 'end':
              return LAYOUT_DICT.end(page >= totalpage)

            case 'curr':
              return LAYOUT_DICT.curr(page)

            case 'info':
              return LAYOUT_DICT.info(total, totalpage)

            case 'jump':
              return LAYOUT_DICT.jump.call(this, page)

            default:
              return ''
          }
        })}
      </div>
    `
  }
}

Pager.reg('pager')
