/**
 * {气泡确认框}
 * @author chensbox<chensbox@foxmail.com>
 * @date 2023/04/28 16:14:10
 */

import { css, html, Component, styleMap } from 'wkit'
import '../form/button.js'

class Popconfirm extends Component {
  static props = {
    title: {
      type: String,
      default: '',
      attribute: false
    },
    confirmButtonText: {
      type: String,
      default: '确定',
      attribute: false
    },
    cancelButtonText: {
      type: String,
      default: '取消',
      attribute: false
    },
    confirmButtonType: {
      type: String,
      default: 'primary',
      attribute: false
    },
    cancelButtonType: {
      type: String,
      default: '',
      attribute: false
    },
    icon: {
      type: String,
      default: 'info',
      attribute: false
    },
    iconColor: {
      type: String,
      default: '#ff9900',
      attribute: false
    },
    hideIcon: {
      type: Boolean,
      default: false
    },

    show: {
      type: Boolean,
      default: false,
      attribute: false
    }
  }

  static styles = [
    css`
      :host {
        position: relative;
        display: inline-block;
      }
      .popover {
        z-index: 10;
        position: fixed;
        padding: 12px;
        min-width: 150px;
        border-radius: 4px;
        border: 1px solid #ebeef5;
        color: #606266;
        line-height: 1.4;
        text-align: left;
        font-size: 14px;
        box-shadow: 0 2px 12px 0 rgba(0, 0, 0, 0.1);
        word-break: break-all;
        background: #fff;
        transition: opacity 0.15s linear;
        .btns {
          display: flex;
          justify-content: flex-end;
          .cancel {
            margin-right: 15px;
            cursor: pointer;
            color: var(--color-teal-2);
          }
        }
      }
      .content {
        padding: 8px 0;
        display: flex;
        align-items: center;
        font-size: 14px;
        color: #606266;
        line-height: 1.4;
        text-align: justify;
        word-break: break-all;
        white-space: nowrap;
        wc-icon {
          margin-right: 5px;
          --wc-icon-size: 14px;
        }
      }

      .arrow {
        position: absolute;
        z-index: 12;
        left: 50%;
        top: 100%;
        transform: translateX(-50%);
        border: 6px solid transparent;
        border-bottom-color: #fff;
        &[top] {
          border-bottom-color: transparent;
          top: 0;
          border-top-color: #fff;
          transform: translateY(-100%);
        }
      }

      .show {
        visibility: visible;
        opacity: 1;
      }
      .hide {
        visibility: hidden;
        opacity: 0;
      }
    `
  ]
  mounted() {
    this.$slot = this.$refs.slot.assignedElements().shift()
    this.$popover = this.$refs.popover
    this.$arrow = this.$refs.arrow
  }

  showPopover() {
    const slotRect = this.$slot.getBoundingClientRect()
    const popoverRect = this.$popover.getBoundingClientRect()
    const halfSlotWidth = slotRect.width / 2
    const halfPopoverWidth = popoverRect.width / 2
    const left = slotRect.left + halfSlotWidth - halfPopoverWidth

    if (left < 0) {
      this.left = slotRect.left
    } else if (left + popoverRect.width > window.innerWidth) {
      this.left = slotRect.right - popoverRect.width
    } else {
      this.left = left
    }

    if (slotRect.bottom + 10 + popoverRect.height > window.innerHeight) {
      this.top = slotRect.top - 10 - popoverRect.height
      this.$arrow.setAttribute('top', '')
    } else {
      this.top = slotRect.bottom + 10
    }

    this.show = true
  }
  hide({ target }) {
    this.show = false
    if (target.hasAttribute('solid')) {
      this.$emit('confirm')
    } else {
      this.$emit('cancel')
    }
  }

  render() {
    let styles = styleMap({
      left: `${this.left}px`,
      top: `${this.top}px`
    })

    return html`
      <div class="popconfirm">
        <slot ref="slot" @click=${this.showPopover}></slot>

        <div class=${`arrow ${this.show ? 'show' : 'hide'}`} ref="arrow"></div>

        <div
          class=${`popover ${this.show ? 'show' : 'hide'}`}
          style=${styles}
          ref="popover"
        >
          <p class="content">
            ${this.hideIcon
              ? ''
              : html`<wc-icon
                  name=${this.icon}
                  style=${`color:${this.iconColor}`}
                ></wc-icon>`}
            ${this.title}
          </p>

          <div class="btns">
            ${!this.cancelButtonType
              ? html`<span class="cancel" @click=${this.hide}
                  >${this.cancelButtonText}</span
                >`
              : html`<wc-button
                  type=${this[cancelButtonType]}
                  class="cancel"
                  size="s"
                  @click=${this.hide}
                  >${this[cancelButtonText]}</wc-button
                >`}

            <wc-button
              type=${this.confirmButtonType}
              size="s"
              solid
              @click=${this.hide}
              >${this.confirmButtonText}</wc-button
            >
          </div>
        </div>
      </div>
    `
  }
}

Popconfirm.reg('popconfirm')
