/**
 * {标签}
 * @author chensbox<chensbox@foxmail.com>
 * @date 2023/04/28 16:14:10
 */

import { css, html, Component } from 'wkit'
import '../icon/index.js'

const ANIMATION = { type: 'scale' }

class Tag extends Component {
  static props = {
    closable: false
  }

  static styles = [
    css`
      :host {
        display: inline-flex;
      }

      .container {
        display: flex;
        align-items: center;
        width: auto;
        height: 32px;
        padding: 0 10px;
        gap: 3px;
        font-size: var(--wc-tag-font, 12px);
        border: 1px solid var(--wc-tag-border-color, var(--color-grey-1));
        border-radius: 3px;
        background: var(--wc-tag-background, var(--color-plain-1));
        white-space: nowrap;
        -webkit-user-select: none;
        user-select: none;
      }
      .close {
        padding: 3px;
        display: inline-flex;
        --wc-icon-size: 8px;
        line-height: 1;
        border-radius: 50%;
        cursor: pointer;

        &:hover {
          background: var(--color-grey-2);
          color: #fff;
        }
      }
    `,
    //尺寸
    css`
      @use 'sass:map';
      $sizes: (
        s: (
          h: 20px,
          p: 0 6px
        ),
        m: (
          h: 24px,
          p: 0 8px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          .container {
            height: map.get($v, 'h');
            padding: map.get($v, 'p');
          }
        }
      }
    `,
    // 配色
    css`
      @use 'sass:map';
      $colors: (
        primary: (
          c: 'teal',
          b: #ecf5ff
        ),
        info: (
          c: 'blue',
          b: #ecf5ff
        ),
        success: (
          c: 'green',
          b: #f0f9eb
        ),
        warning: (
          c: 'orange',
          b: #fdf6ec
        ),
        danger: (
          c: 'red',
          b: #fef0f0
        )
      );

      @loop $t, $v in $colors {
        :host([type='#{$t}']) {
          .container {
            border-color: var(--color-#{map.get($v, 'c')}-a);
            background: #{map.get($v, 'b')};
            color: var(--color-#{map.get($v, 'c')}-3);
          }
          .close:hover {
            color: #fff;
            background: var(--color-#{map.get($v, 'c')}-1);
          }
        }
      }
    `
  ]

  #close() {
    this.$refs.tag.$animate(true).then(_ => this.remove())
    this.$emit('close')
  }

  render() {
    return html`
      <main class="container" ref="tag" #animation=${ANIMATION}>
        <slot></slot>
        ${this.closable
          ? html`<span class="close" @click=${this.#close}
              ><wc-icon name="close"></wc-icon
            ></span>`
          : ''}
      </main>
    `
  }
}

Tag.reg('tag')
