/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/21 16:14:10
 */

import { css, html, Component, bind, styleMap, offset } from 'wkit'

class Tooltip extends Component {
  static props = {
    title: 'str!'
  }

  static styles = [
    css`
      :host {
        display: inline-flex;
      }

      .container {
        position: relative;
      }

      .tooltip {
        display: none;
        position: fixed;
        z-index: 9;
        justify-content: center;
        align-items: center;
        max-width: 360px;
        min-width: 32px;
        padding: 6px 8px;
        border-radius: 3px;
        font-size: var(--wc-tooltip-font, 14px);
        background: var(--wc-tooltip-background, #fff);
        color: var(--wc-tooltip-color, var(--color-dark-1));
        box-shadow: 0 0 3px var(--wc-tooltip-shadow, rgba(0, 0, 0, 0.2));
        word-break: break-all;
        -webkit-user-select: none;
        user-select: none;

        &::after {
          position: absolute;
          display: block;
          width: 8px;
          height: 8px;
          border-radius: 2px;
          background: var(--wc-tooltip-background, #fff);
          content: '';
          transform: rotate(45deg);
          box-shadow: -1px -1px 0 var(--wc-tooltip-shadow, rgba(0, 0, 0, 0.1));
        }

        &[placement='left-top'] {
          &::after {
            right: 16px;
            bottom: -4px;
            box-shadow: 1px 1px 0 var(--wc-tooltip-shadow, rgba(0, 0, 0, 0.1));
          }
        }
        &[placement='left-bottom'] {
          &::after {
            right: 16px;
            top: -4px;
          }
        }
        &[placement='right-top'] {
          &::after {
            left: 16px;
            bottom: -4px;
            box-shadow: 1px 1px 0 var(--wc-tooltip-shadow, rgba(0, 0, 0, 0.1));
          }
        }
        &[placement='right-bottom'] {
          &::after {
            left: 16px;
            top: -4px;
          }
        }
      }
    `
  ]

  mounted() {
    bind(this.$refs.wrap, 'mouseenter', ev => {
      let { left, top } = offset(this)
      let placement = 'left'
      let styles = { display: 'block' }

      if (left < 360 || (left > 360 && window.innerWidth - left > 360)) {
        placement = 'right'
        styles.left = left + 'px'
      } else {
        let right = window.innerWidth - left - this.clientWidth
        styles.right = right + 'px'
      }

      if (top < 96) {
        top += 8 + this.clientHeight
        placement += '-bottom'
        styles.top = top + 'px'
      } else {
        let bottom = window.innerHeight - top + 8
        placement += '-top'
        styles.bottom = bottom + 'px'
      }
      //
      this.$refs.tips.setAttribute('placement', placement)
      this.$refs.tips.style.cssText = styleMap(styles)
      this.$refs.tips.$animate()
    })
    bind(this.$refs.wrap, 'mouseleave', ev => {
      this.$refs.tips.$animate(true)
    })
  }

  render() {
    return html`
      <main class="container">
        <div class="wrapper" ref="wrap"><slot></slot></div>
        <div class="tooltip" ref="tips" #animation=${{}}>
          <slot name="title">${this.title}</slot><i class="trigon"></i>
        </div>
      </main>
    `
  }
}

Tooltip.reg('tooltip')
